/*****************************************************************************\
*       This is a part of the Microsoft Source Code Samples.
*       Copyright (C) 1992-1995 Microsoft Corporation.
*       All rights reserved.
*       This source code is only intended as a supplement to
*       Microsoft Development Tools and/or WinHelp documentation.
*       See these sources for detailed information regarding the
*       Microsoft samples programs.
\*****************************************************************************/
/*****************************************************************************
 *																			 *
 *   PROGRAM: FastBlt.c 													 *
 *																			 *
 *   PURPOSE: FastBlt                                       				 *
 *																			 *
 *   FUNCTIONS:																 *
 *																			 *
 *   GetLastErrorBox() - Report GetLastError() values as text                *
 *   WinMain() - calls initialization function, processes message loop		 *
 *   InitApplication() - initializes window data and registers window		 *
 *   InitInstance() - saves instance handle and creates main window			 *
 *   WndProc() - processes messages											 *
 *   CenterWindow() - used to center the "About" box over application window *
 *   About() - processes messages for "About" dialog box					 *
 *																			 *
 *****************************************************************************/

#include <windows.h>   // required for all Windows applications
#include "dib.h" 
#include "palette.h" 
#include "resource.h"   // specific to this program
#include "fastblt.h"   // specific to this program

HINSTANCE hInst;          // current instance
HICON hIcon = NULL;
HANDLE ghWnd;

char szAppName[] = "FastBlt";   // The name of this application
char szTitle[]   = "Fast BitBlt Demo"; // The title bar text

BOOL InitApplication(HANDLE);
BOOL InitInstance(HANDLE, int);
LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK About  (HWND, UINT, WPARAM, LPARAM);

/**********************************************************************
 *                                                                    *
 * FUNCTION:  GetLastErrorBox(HWND, LPSTR)                            *  
 *																	  *
 * PURPOSE:   Gets the error status and, if an error is indicated,    *
 *            converts the error number into text and displays it     *
 *            in a MessageBox.                                        *
 *                                                                    *
 *********************************************************************/
DWORD GetLastErrorBox(HWND hWnd, LPSTR lpTitle)
{
   LPVOID lpv;
   DWORD  dwRv;

   if (GetLastError() == 0) return 0;
   
   dwRv = FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER |
                        FORMAT_MESSAGE_FROM_SYSTEM,
                        NULL,
                        GetLastError(),
                        MAKELANGID(LANG_ENGLISH, SUBLANG_ENGLISH_US),
                        (LPVOID)&lpv,
                        0,
                        NULL);
   
   MessageBox(hWnd, lpv, lpTitle, MB_OK);
   
   if(dwRv)
      LocalFree(lpv);
   
   SetLastError(0);
   return dwRv;
}

/*****************************************************************************
 *																			 *
 * FUNCTION: WinMain(HINSTANCE, HINSTANCE, LPSTR, int)						 *
 *																			 *
 * PURPOSE: calls initialization function, processes message loop			 *
 *																			 *
 * COMMENTS:																 *
 *																			 *
 *    Windows recognizes this function by name as the initial entry point	 *
 *    for the program.  This function calls the application initialization	 *
 *    routine, if no other instance of the program is running, and always	 *
 *    calls the instance initialization routine.  It then executes a message *
 *    retrieval and dispatch loop that is the top-level control structure	 *
 *    for the remainder of execution.  The loop is terminated when a WM_QUIT *
 *    message is received, at which time this function exits the application *
 *    instance by returning the value passed by PostQuitMessage().			 *
 *																			 *
 *    If this function must abort before entering the message loop, it		 *
 *    returns the conventional value NULL.									 *
 *																			 *
 ****************************************************************************/
int APIENTRY WinMain(
        HINSTANCE hInstance,
        HINSTANCE hPrevInstance,
        LPSTR lpCmdLine,
        int nCmdShow)
{		
    MSG msg;
    HANDLE hAccelTable;
	
    if (!hPrevInstance) {       // Other instances of app running?
            if (!InitApplication(hInstance)) { // Initialize shared things
            return (FALSE);     // Exits if unable to initialize
        }
    }

    /* Perform initializations that apply to a specific instance */

    if (!InitInstance(hInstance, nCmdShow)) {
            return (FALSE);
    }

    hAccelTable = LoadAccelerators (hInstance, "TEXTFXACCEL");

    /* Acquire and dispatch messages until a WM_QUIT message is received. */

    while (GetMessage(&msg, // message structure
       NULL,   // handle of window receiving the message
       0,      // lowest message to examine
       0))     // highest message to examine
    {
        if (!TranslateAccelerator (msg.hwnd, hAccelTable, &msg)) {
            TranslateMessage(&msg);// Translates virtual key codes
            DispatchMessage(&msg); // Dispatches message to window
        }
    }

    return (msg.wParam); // Returns the value from PostQuitMessage

    UNREFERENCED_PARAMETER(lpCmdLine); // This will prevent 'unused formal parameter' warnings
}


/*****************************************************************************
 *																			 *
 *  FUNCTION: InitApplication(HINSTANCE)									 *
 *																			 *
 *  PURPOSE: Initializes window data and registers window class				 *
 *																			 *
 *  COMMENTS:																 *
 *																			 *
 *    This function is called at initialization time only if no other		 *
 *    instances of the application are running.  This function performs	     *
 *    initialization tasks that can be done once for any number of running	 *
 *    instances.															 *
 *																			 *
 *    In this case, we initialize a window class by filling out a data		 *
 *    structure of type WNDCLASS and calling the Windows RegisterClass()	 *
 *    function.  Since all instances of this application use the same window *
 *    class, we only need to do this when the first instance is initialized. *
 *																			 *
 *****************************************************************************/
BOOL InitApplication(HINSTANCE hInstance)
{
    WNDCLASS  wc;

    // Fill in window class structure with parameters that describe the
    // main window.

    wc.style         = 0;                      // Class style(s).
    wc.lpfnWndProc   = (WNDPROC)WndProc;       // Window Procedure
    wc.cbClsExtra    = 0;                      // No per-class extra data.
    wc.cbWndExtra    = 0;                      // No per-window extra data.
    wc.hInstance     = hInstance;              // Owner of this class
    wc.hIcon         = LoadIcon (hInstance, szAppName); // Icon name from .RC
    wc.hCursor       = LoadCursor(NULL, IDC_ARROW);// Cursor
    wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);// Default color
    wc.lpszMenuName  = szAppName;           // Menu name from .RC
    wc.lpszClassName = szAppName;              // Name to register as

    // Register the window class and return success/failure code.
    return (RegisterClass(&wc));
}


/*****************************************************************************
 *																			 *
 *   FUNCTION:  InitInstance(HINSTANCE, int)								 *
 *																			 *
 *   PURPOSE:  Saves instance handle and creates main window				 *
 *																			 *
 *   COMMENTS:																 *
 *																			 *
 *      This function is called at initialization time for every instance of *
 *      this application.  This function performs initialization tasks that	 *
 *      cannot be shared by multiple instances.								 *
 *																			 *
 *      In this case, we save the instance handle in a static variable and	 *
 *      create and display the main program window.							 *
 *																			 *
 ****************************************************************************/
BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
    HWND            hWnd; // Main window handle.

    // Save the instance handle in static variable, which will be used in
    // many subsequence calls from this application to Windows.

    hInst = hInstance; // Store instance handle in our global variable

    // Create a main window for this application instance.

    hWnd = CreateWindow(
            szAppName,           // See RegisterClass() call.
            szTitle,             // Text for window title bar.
            WS_OVERLAPPEDWINDOW,// Window style.
            CW_USEDEFAULT, 0, CW_USEDEFAULT, 0, // Use default positioning
            NULL,                // Overlapped windows have no parent.
            NULL,                // Use the window class menu.
            hInstance,           // This instance owns this window.
            NULL                 // We don't use any data in our WM_CREATE
    );

    // If window could not be created, return "failure"
    if (!hWnd) {
            return (FALSE);
    }

    // Make the window visible; update its client area; and return "success"
    ShowWindow(hWnd, nCmdShow); // Show the window
    UpdateWindow(hWnd);         // Sends WM_PAINT message

    return (TRUE);              // We succeeded...

}
									  
HBITMAP DIBSectionFromDIB(HANDLE hInputDIB) 
{
    HANDLE hDIB;
    HBITMAP hbm;
    LPBITMAPINFO bi;
    RGBQUAD *rgbq;
    LPBYTE lpSrcBits, lpDstBits;

    hDIB = ChangeDIBFormat(hInputDIB,  24, BI_RGB);
    if (!hDIB) {
        GlobalFree(hInputDIB);
        return NULL;
    }
    GetDIBPointers(hDIB, &bi, &rgbq, &lpSrcBits);

    hbm = CreateDIBSection(NULL, bi, DIB_RGB_COLORS, &lpDstBits, NULL, (DWORD)0); 

    if (!hbm) {
        GlobalFree(hInputDIB);
        return NULL;
    }

    CopyMemory(lpDstBits, lpSrcBits, BYTESPERLINE(bi->bmiHeader.biWidth, bi->bmiHeader.biBitCount) * bi->bmiHeader.biHeight);
    
    GlobalFree(hDIB);

    return hbm;
}

/****************************************************************************

        FUNCTION: WndProc(HWND, UINT, WPARAM, LPARAM)

        PURPOSE:  Processes messages

        MESSAGES:

        WM_COMMAND    - application menu (About dialog box)
        WM_DESTROY    - destroy window

        COMMENTS:

        To process the IDM_ABOUT message, call MakeProcInstance() to get the
        current instance address of the About() function.  Then call Dialog
        box which will create the box according to the information in your
        FASTBLT.rc file and turn control over to the About() function.  When
        it returns, free the intance address.

****************************************************************************/

LRESULT CALLBACK WndProc(
                HWND hWnd,         // window handle
                UINT message,      // type of message
                WPARAM uParam,     // additional information
                LPARAM lParam)     // additional information
{
	int wmId, wmEvent;
    static char szFilename[MAX_PATH] = {0};
	static HBITMAP hbm = NULL;
    static DWORD dwGDITime = 0;
    static DWORD dwFastBltTime = 0;

	switch (message) {
        case WM_PAINT:
            {
               PAINTSTRUCT ps;
               HDC hDC = BeginPaint(hWnd, &ps);
               char szGDIText[128];
               char szFastBltText[128];
               SIZE size, size2;
               RECT rect;

               if (dwGDITime)
                 wsprintf(szGDIText, "Elapsed time for BitBlt() operation: %lums", dwGDITime);
               else
                 wsprintf(szGDIText, "Elapsed time for BitBlt() operation: <Untested>");
               
               if (dwFastBltTime)
                 wsprintf(szFastBltText, "Elapsed time for DIBSecBitBlt() operation: %lums", dwFastBltTime);
               else
                 wsprintf(szFastBltText, "Elapsed time for DIBSecBitBlt() operation: <Untested>");

               GetTextExtentPoint32(hDC, szGDIText, strlen(szGDIText), &size);
               GetTextExtentPoint32(hDC, szFastBltText, strlen(szFastBltText), &size2);

               SetRect(&rect, 0, 0, max(size.cx, size2.cx)+20, (max(size.cy, size2.cy) << 1)+20);

               DrawEdge(hDC, &rect, BDR_RAISEDINNER, BF_MIDDLE | BF_RECT); 

               SetBkMode(hDC, TRANSPARENT);
               TextOut(hDC, 6, 6, szGDIText, strlen(szGDIText));
               TextOut(hDC, 6,size.cy+10, szFastBltText, strlen(szFastBltText));

               EndPaint(hWnd, &ps);
            }
            break;

        case WM_COMMAND:
            wmId    = LOWORD(uParam);
            wmEvent = HIWORD(uParam);

            switch (wmId) {
	            case IDM_LOADDIB:
                        {
                            HANDLE hDIB;
                            OPENFILENAME ofn;
                            char szNewTitle[MAX_PATH];
                            char szDirName[MAX_PATH];
                            
                            /* Get the windows directory name, and store in szDirName */
                            GetWindowsDirectory(szDirName, sizeof(szDirName));
                            
                            /* Set all structure members to zero. */
                            ZeroMemory(&ofn, sizeof(OPENFILENAME));

                            ofn.lStructSize = sizeof(OPENFILENAME);
                            ofn.hwndOwner = hWnd;
                            ofn.lpstrFilter = "BMP Files (*.BMP)\0 *.BMP\0\0";
                            ofn.lpstrFile = szFilename;
                            ofn.nMaxFile = sizeof(szFilename);
                            ofn.lpstrInitialDir = szDirName;
                            ofn.Flags = OFN_SHOWHELP | OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST;

                            if (GetOpenFileName(&ofn)) {
                               hDIB = OpenDIB(szFilename);   
                               if (!hDIB)
                                  MessageBox(hWnd, "Unable to load file!", szFilename, MB_ICONSTOP);
                            } else {
                                if (strlen(szFilename) != 0)
                                  MessageBox(hWnd, "Unable to load file!", szFilename, MB_ICONSTOP);
                                
                                return 0;
                            }
                             
                            // if we already have a bitmap, then get rid of it
                            if (hbm)
                              DeleteObject(hbm);

                            // Convert the DIB into a DIB section
                            hbm = DIBSectionFromDIB(hDIB);

                            strcpy(szNewTitle, szTitle);
                            strcat(szNewTitle, " - ");
                            strcat(szNewTitle, szFilename);
                            SetWindowText(hWnd, szNewTitle);

                            dwGDITime = 0;      
                            dwFastBltTime = 0;  

                            EnableMenuItem (GetMenu(hWnd), IDM_DEMOGDI, MF_ENABLED);
                            EnableMenuItem (GetMenu(hWnd), IDM_DEMOFASTBLT, MF_ENABLED);
                        }
                        break;
	
		 		case IDM_DEMOGDI:
                case IDM_DEMOFASTBLT:
                        {
                            HDC hDC = GetDC(hWnd);
                            HDC hMemDC = CreateCompatibleDC(hDC);
                            HPALETTE hPal = CreateHalftonePalette(hDC);
                            BITMAP bm;
                            DWORD dwTime;
                            RECT rect; 

                            GetObject(hbm, sizeof(BITMAP), &bm);

                            SelectObject(hMemDC, hbm);
                            SelectPalette(hDC, hPal, FALSE);
                            RealizePalette(hDC);

                            GetClientRect(hWnd, &rect);
                            PatBlt(hDC, 0, 0, rect.right, rect.bottom, WHITENESS);

                            dwTime = GetTickCount();
                            if (wmId == IDM_DEMOGDI)
                                BitBlt(hDC, 0, 0, bm.bmWidth, bm.bmHeight, hMemDC, 0, 0, SRCCOPY);
                            else
                                DIBSecBitBlt(hDC, 0, 0, bm.bmWidth, bm.bmHeight, hMemDC, 0, 0, SRCCOPY);
                            GdiFlush();
                            dwTime = GetTickCount() - dwTime;

                            if (wmId == IDM_DEMOGDI)
                                dwGDITime = dwTime;
                            else
                                dwFastBltTime = dwTime;
                            
                            DeleteDC(hMemDC);
                            ReleaseDC(hWnd, hDC);
                            DeleteObject(hPal);

                            InvalidateRect(hWnd, NULL, FALSE);
                        }
                        break;

                case IDM_ABOUT:
                    DialogBox(hInst,          // current instance
                             "ABOUTBOX",      // dlg resource to use
                             hWnd,            // parent handle
                             (DLGPROC)About); // About() instance address

                    break;
                
                case IDM_EXIT:
                        DestroyWindow (hWnd);
                        break;

                default:
                        return (DefWindowProc(hWnd, message, uParam, lParam));
            }
            break;

        case WM_DESTROY:  // message: window being destroyed
                PostQuitMessage(0);
                break;
    
        default:          // Passes it on if unproccessed
                return (DefWindowProc(hWnd, message, uParam, lParam));
	}
	return (0);
}

/*****************************************************************************
 *																			 *
 *   FUNCTION: CenterWindow (HWND, HWND)									 *
 *																			 *
 *   PURPOSE:  Center one window over another								 *
 *																			 *
 *   COMMENTS:																 *
 *																			 *
 *   Dialog boxes take on the screen position that they were designed at,	 *
 *   which is not always appropriate. Centering the dialog over a particular *
 *   window usually results in a better position.							 *
 *																			 *
 ****************************************************************************/

BOOL CenterWindow (HWND hwndChild, HWND hwndParent)
{
        RECT    rChild, rParent;
        int     wChild, hChild, wParent, hParent;
        int     wScreen, hScreen, xNew, yNew;
        HDC     hdc;

        // Get the Height and Width of the child window
        GetWindowRect (hwndChild, &rChild);
        wChild = rChild.right - rChild.left;
        hChild = rChild.bottom - rChild.top;

        // Get the Height and Width of the parent window
        GetWindowRect (hwndParent, &rParent);
        wParent = rParent.right - rParent.left;
        hParent = rParent.bottom - rParent.top;

        // Get the display limits
        hdc = GetDC (hwndChild);
        wScreen = GetDeviceCaps (hdc, HORZRES);
        hScreen = GetDeviceCaps (hdc, VERTRES);
        ReleaseDC (hwndChild, hdc);

        // Calculate new X position, then adjust for screen
        xNew = rParent.left + ((wParent - wChild) /2);
        if (xNew < 0) {
                xNew = 0;
        } else if ((xNew+wChild) > wScreen) {
                xNew = wScreen - wChild;
        }

        // Calculate new Y position, then adjust for screen
        yNew = rParent.top  + ((hParent - hChild) /2);
        if (yNew < 0) {
                yNew = 0;
        } else if ((yNew+hChild) > hScreen) {
                yNew = hScreen - hChild;
        }

        // Set it, and return
        return SetWindowPos (hwndChild, NULL,
                xNew, yNew, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
}


/*****************************************************************************
 *																			 *
 *   FUNCTION: About(HWND, UINT, WPARAM, LPARAM)							 *
 *																			 *
 *   PURPOSE:  Processes messages for "About" dialog box					 *
 *																			 *
 *   MESSAGES:																 *
 *																			 *
 *   WM_INITDIALOG - initialize dialog box									 *
 *   WM_COMMAND    - Input received											 *
 *																			 *
 *   COMMENTS:																 *
 *																			 *
 *   Display version information from the version section of the			 *
 *   application resource.													 *
 *																			 *
 *   Wait for user to click on "Ok" button, then close the dialog box.		 *
 *																			 *
 ****************************************************************************/

LRESULT CALLBACK About(
                HWND hDlg,           // window handle of the dialog box
                UINT message,        // type of message
                WPARAM uParam,       // message-specific information
                LPARAM lParam)
{
    switch (message) {
        case WM_INITDIALOG:  // message: initialize dialog box
            // Center the dialog over the application window
            CenterWindow (hDlg, GetWindow (hDlg, GW_OWNER));
            return (TRUE);

        case WM_COMMAND:                      // message: received a command
            if (LOWORD(uParam) == IDOK || LOWORD(uParam) == IDCANCEL) { 
                EndDialog(hDlg, TRUE);        // Exit the dialog
                return (TRUE);
            }
            break;
    }
    return (FALSE); // Didn't process the message

    UNREFERENCED_PARAMETER(lParam); // This will prevent 'unused formal parameter' warnings
}
