#include "GlideAlternative.h"
#include <string.h>
#include <windows.h>
#include <mmsystem.h>

GrScreenResolution_t scrResolution;
GrColorFormat_t cFormat=GR_COLORFORMAT_ABGR;
GrColor_t ConstColor=0xFFFFFFFF;
GrCoordinateSpaceMode_t CoordMode=GR_WINDOW_COORDS;

FxU32 ClipMinX,ClipMinY,ClipMaxX,ClipMaxY;
int ParamOffset[128];

FxU32 PointsCount=0;
FxU32 LinesCount=0;

HDC ScrDC;
HWND hWnd;
HDC hDC;
HDC BackDC;
HBITMAP BackBmp;

Size Resolutions[17]={
	{320,200},{320,240},{400,256},{512,384},{640,200},{640,350},
	{640,400},{640,480},{800,600},{960,720},{856,480},{512,256},
	{1024,768},{1280,1024},{1600,1200},{400,300},{1152,864}
};

const int VideoMemory=16*1048576;

inline unsigned ColorRGB(int r,int g,int b) {
	return (b<<16)|(g<<8)|r;
}

inline float GetWinX(const void *vtx) {
	if (CoordMode)
		return Resolutions[scrResolution].Width*(1+(*((float*)vtx))/(*((float*)vtx+(ParamOffset[GR_PARAM_W]>>2))))/2;
	else
		return *((float*)vtx);
}

inline float GetWinY(const void *vtx) {
	if (CoordMode)
		return Resolutions[scrResolution].Height*(1+(*((float*)vtx+1))/(*((float*)vtx+(ParamOffset[GR_PARAM_W]>>2))))/2;
	else
		return *((float*)vtx+1);
}

inline unsigned ColorConv(GrColor_t c) {
	switch (cFormat) {
	case GR_COLORFORMAT_ARGB:
		return (c&0xFF00FF00)|((c&0xFF)<<16)|((c>>16)&0xFF);
	case GR_COLORFORMAT_ABGR:
		return c;
	case GR_COLORFORMAT_RGBA:
		return ((c&0xFF)<<24)|(((c>>8)&0xFF)<<16)|(((c>>16)&0xFF)<<8)|(c>>24);
	case GR_COLORFORMAT_BGRA:
		return ((c>>8)&0xFFFFFF)|((c&0xFF)<<24);
	}
	return 0;
}

/*
** rendering functions
*/
FX_ENTRY void FX_CALL grDrawPoint( const void *pt ) {
	float *x,*y;
	x=(float*)pt; y=x+1;
	unsigned c=ConstColor;
	int os;
	if (os=ParamOffset[GR_PARAM_RGB]) {
		float r,g,b;
		r=*(x+os);
		g=*(x+os+1);
		b=*(x+os+2);
		c=ColorRGB(r*255.0f,g*255.0f,b*255.0f);
	}
	if (os=ParamOffset[GR_PARAM_PARGB]) c=ColorConv(*((unsigned*)pt+os));
	SetPixelV(BackDC,GetWinX(pt),GetWinY(pt),c&0xFFFFFF);
	PointsCount++;
};

FX_ENTRY void FX_CALL grDrawLine( const void *v1, const void *v2 ) {
	float *x1,*y1,*x2,*y2;
	x1=(float*)v1; y1=x1+1;
	x2=(float*)v2; y2=x2+1;
	unsigned c=ConstColor;
	int os;
	if (os=ParamOffset[GR_PARAM_RGB]) {
		float r,g,b;
		r=(*(x1+os)+*(x2+os))/2; if (r>1) r=1;
		g=(*(x1+os+1)+*(x2+os+1))/2; if (g>1) g=1;
		b=(*(x1+os+2)+*(x2+os+2))/2; if (b>1) b=1;
		c=ColorRGB(r*255.0f,g*255.0f,b*255.0f);
	}
	if (os=ParamOffset[GR_PARAM_PARGB]) {
		c=ColorConv(*((unsigned*)v1+os));
	}
	HGDIOBJ hObj;
	hObj=CreatePen(PS_SOLID,0,c&0xFFFFFF);
	SelectObject(BackDC,hObj);
	MoveToEx(BackDC,GetWinX(v1),GetWinY(v1),0);
	LineTo(BackDC,GetWinX(v2),GetWinX(v2));
	DeleteObject(hObj);
	LinesCount++;
};

FX_ENTRY void FX_CALL grDrawTriangle( const void *v1, const void *v2, const void *v3 ) {
	HGDIOBJ hObj1,hObj2;
	POINT p[3];
	float *x1,*y1,*x2,*y2,*x3,*y3;
	x1=(float*)v1; y1=x1+1; p[0].x=GetWinX(v1); p[0].y=GetWinY(v1);
	x2=(float*)v2; y2=x2+1; p[1].x=GetWinX(v2); p[1].y=GetWinY(v2);
	x3=(float*)v3; y3=x3+1; p[2].x=GetWinX(v3); p[2].y=GetWinY(v3);
	unsigned c=ConstColor;
	int os;
	if (os=ParamOffset[GR_PARAM_RGB]) {
		float r,g,b;
		r=(*(x1+os)+*(x2+os)+*(x3+os))/3; if (r>1) r=1;
		g=(*(x1+os+1)+*(x2+os+1)+*(x3+os+1))/3; if (g>1) g=1;
		b=(*(x1+os+2)+*(x2+os+2)+*(x3+os+2))/3; if (b>1) b=1;
		c=ColorRGB(r*255.0f,g*255.0f,b*255.0f);
	}
	if (os=ParamOffset[GR_PARAM_PARGB]) {
		c=ColorConv(*((unsigned*)v1+os));
	}
	SelectObject(BackDC,hObj1=CreateSolidBrush(c&0xFFFFFF));
	SelectObject(BackDC,hObj2=CreatePen(PS_NULL,0,0));
	Polygon(BackDC,p,3);
	DeleteObject(hObj1);
	DeleteObject(hObj2);
};

FX_ENTRY void FX_CALL grVertexLayout(FxU32 param, FxI32 offset, FxU32 mode) {
	if (param>0&&param<128)
		switch (mode) {
		case GR_PARAM_ENABLE:
			ParamOffset[param]=offset>>2;
			break;
		case GR_PARAM_DISABLE:
			ParamOffset[param]=0;
			break;
		}
};

FX_ENTRY void FX_CALL grDrawVertexArray(FxU32 mode, FxU32 Count, void *pointers[]) {
	
};

FX_ENTRY void FX_CALL grDrawVertexArrayContiguous(FxU32 mode, FxU32 Count, void *pointers, FxU32 stride) {
	FxU32 i;
	float *x0,*y0;
	x0=(float*)pointers;
	y0=x0+1;
	POINT *p=new POINT [Count];
	for (i=0;i<Count;i++) {
		p[i].x=GetWinX(x0+i*stride/4);
		p[i].y=GetWinY(x0+i*stride/4);
	}
	unsigned c=ConstColor;
	int os;
	if (os=ParamOffset[GR_PARAM_RGB]) {
		float r,g,b;
		r=*(x0+os); if (r>1) r=1;
		g=*(x0+os+1); if (g>1) g=1;
		b=*(x0+os+2); if (b>1) b=1;
		c=ColorRGB(r*255.0f,g*255.0f,b*255.0f);
	}
	if (os=ParamOffset[GR_PARAM_PARGB])
		c=ColorConv(*((unsigned*)pointers+os));
	HGDIOBJ hObj,hObj1,hObj2;
	switch (mode) {		// TRIANGLE_STRIP, TRIANGLE_FAN  
	case GR_LINE_STRIP:
		hObj=CreatePen(PS_SOLID,0,c&0xFFFFFF);
		SelectObject(BackDC,hObj);
		Polyline(BackDC,p,Count);
		DeleteObject(hObj);
		LinesCount+=Count-1;
		break;
	case GR_LINES:
		hObj=CreatePen(PS_SOLID,0,c&0xFFFFFF);
		SelectObject(BackDC,hObj);
		for (i=0;i<=Count-2;i+=2) Polyline(BackDC,p+i,2);
		DeleteObject(hObj);
		LinesCount+=Count/2;
		break;
	case GR_TRIANGLES:
		SelectObject(BackDC,hObj1=CreateSolidBrush(c&0xFFFFFF));
		SelectObject(BackDC,hObj2=CreatePen(PS_NULL,0,0));
		for (i=0;i<=Count-3;i+=3) Polygon(BackDC,p+i,3);
		DeleteObject(hObj1);
		DeleteObject(hObj2);
		break;
	case GR_POLYGON:
		SelectObject(BackDC,hObj1=CreateSolidBrush(c&0xFFFFFF));
		SelectObject(BackDC,hObj2=CreatePen(PS_NULL,0,0));
		Polygon(BackDC,p,Count);
		DeleteObject(hObj1);
		DeleteObject(hObj2);
		break;
	}
};

/*
**  Antialiasing Functions
*/

FX_ENTRY void FX_CALL grAADrawTriangle(
                 const void *a, const void *b, const void *c,
                 FxBool ab_antialias, FxBool bc_antialias, FxBool ca_antialias
                 ) {
	grDrawTriangle(a,b,c);		//  
};

/*
** buffer management
*/
FX_ENTRY void FX_CALL grBufferClear( GrColor_t color, GrAlpha_t alpha, FxU32 depth ) {
	SelectClipRgn(BackDC,CreateRectRgn(ClipMinX,ClipMinY,ClipMaxX,ClipMaxY));
	HGDIOBJ hObj1,hObj2;
	SelectObject(BackDC,hObj1=CreateSolidBrush(color&0xFFFFFF));
	SelectObject(BackDC,hObj2=CreatePen(PS_NULL,0,0));
	Rectangle(BackDC,ClipMinX,ClipMinY,ClipMaxX-ClipMinX+1,ClipMaxY-ClipMinY+1);
	DeleteObject(hObj1);
	DeleteObject(hObj2);
};

FX_ENTRY void FX_CALL grBufferSwap( FxU32 swap_interval ) {
	BitBlt(hDC,ClipMinX,ClipMinY,ClipMaxX-ClipMinX+1,ClipMaxY-ClipMinY+1,BackDC,0,0,SRCCOPY);
};

FX_ENTRY void FX_CALL grRenderBuffer( GrBuffer_t buffer ) {
	
};

/*
** error management
*/
typedef void (*GrErrorCallbackFnc_t)( const char *string, FxBool fatal );

FX_ENTRY void FX_CALL grErrorSetCallback( GrErrorCallbackFnc_t fnc ) {
	
};

/*
** SST routines
*/
FX_ENTRY void FX_CALL grFinish(void) {
	
};

FX_ENTRY void FX_CALL grFlush(void) {
	GdiFlush();
};

FX_ENTRY GrContext_t FX_CALL grSstWinOpen(
          FxU32                HWnd,
          GrScreenResolution_t screen_resolution,
          GrScreenRefresh_t    refresh_rate,
          GrColorFormat_t      color_format,
          GrOriginLocation_t   origin_location,
          int                  nColBuffers,
          int                  nAuxBuffers) {
	/*if (screen_resolution<0||screen_resolution>16) return 0;
	if (refresh_rate<0||refresh_rate>8) return 0;
	if (!(color_format&0xFFFFFFFC)) return 0;
	if (!(origin_location&0xFFFFFFFE)) return 0;
	if (nColBuffers<1||nColBuffers>3) return 0;
	if (!(nAuxBuffers&0xFFFFFFFE)) return 0;*/
	scrResolution=screen_resolution;
	cFormat=color_format;
	hWnd=(HWND)HWnd;
	Sleep(1000);
	hWnd=GetForegroundWindow();
	//hWnd=GetNextWindow(hWnd,GW_HWNDNEXT);
	hDC=GetDC(hWnd);
	ScrDC=CreateDC("DISPLAY","","",0);
	//BackDC=GetDC(GetNextWindow(hWnd,GW_HWNDPREV));
	BackDC=CreateCompatibleDC(ScrDC);
	BackBmp=CreateBitmap(Resolutions[scrResolution].Width,Resolutions[scrResolution].Height,1,32,NULL);
	SelectObject(BackDC,BackBmp);

	BackDC=hDC;

	ClipMinX=ClipMinY=0;
	ClipMaxX=Resolutions[scrResolution].Width-1;
	ClipMaxY=Resolutions[scrResolution].Height-1;

	//sndPlaySound("Glide3x.wav",0);

	return HWnd;
};

FX_ENTRY FxBool FX_CALL grSstWinClose( GrContext_t context ) {
	return 0;	
};

FX_ENTRY FxBool FX_CALL grSelectContext( GrContext_t context ) {
	return 0;
};

FX_ENTRY void FX_CALL grSstOrigin(GrOriginLocation_t  origin) {

};

FX_ENTRY void FX_CALL grSstSelect( int which_sst ) {

};

/*
** Glide configuration and special effect maintenance functions
*/
FX_ENTRY void FX_CALL grAlphaBlendFunction(
                     GrAlphaBlendFnc_t rgb_sf,   GrAlphaBlendFnc_t rgb_df,
                     GrAlphaBlendFnc_t alpha_sf, GrAlphaBlendFnc_t alpha_df
                     ) {
	
};

FX_ENTRY void FX_CALL grAlphaCombine(
               GrCombineFunction_t function, GrCombineFactor_t factor,
               GrCombineLocal_t local, GrCombineOther_t other,
               FxBool invert
               ) {
	
};

FX_ENTRY void FX_CALL grAlphaControlsITRGBLighting( FxBool enable ) {
	
};

FX_ENTRY void FX_CALL grAlphaTestFunction( GrCmpFnc_t function ) {
	
};

FX_ENTRY void FX_CALL grAlphaTestReferenceValue( GrAlpha_t value ) {
	
};

FX_ENTRY void FX_CALL grChromakeyMode( GrChromakeyMode_t mode ) {
	
};

FX_ENTRY void FX_CALL grChromakeyValue( GrColor_t value ) {
	
};

FX_ENTRY void FX_CALL grClipWindow( FxU32 minx, FxU32 miny, FxU32 maxx, FxU32 maxy ) {
	ClipMinX=minx;
	ClipMinY=miny;
	ClipMaxX=maxx;
	ClipMaxY=maxy;
	FxU32 w=Resolutions[scrResolution].Width-1;
	FxU32 h=Resolutions[scrResolution].Height-1;
	if (ClipMinX<0) ClipMinX=0;
	if (ClipMinY<0) ClipMinY=0;
	if (ClipMinX<w) ClipMinX=w;
	if (ClipMinY<h) ClipMinY=h;
	
};

FX_ENTRY void FX_CALL grColorCombine(
               GrCombineFunction_t function, GrCombineFactor_t factor,
               GrCombineLocal_t local, GrCombineOther_t other,
               FxBool invert ) {
	
};

FX_ENTRY void FX_CALL grColorMask( FxBool rgb, FxBool a ) {
	
};

FX_ENTRY void FX_CALL grCullMode( GrCullMode_t mode ) {
	
};

FX_ENTRY void FX_CALL grConstantColorValue( GrColor_t value ) {
	ConstColor=ColorConv(value);
};

FX_ENTRY void FX_CALL grDepthBiasLevel( FxI32 level ) {
	
};

FX_ENTRY void FX_CALL grDepthBufferFunction( GrCmpFnc_t function ) {
	
};

FX_ENTRY void FX_CALL grDepthBufferMode( GrDepthBufferMode_t mode ) {
	
};

FX_ENTRY void FX_CALL grDepthMask( FxBool mask ) {
	
};

FX_ENTRY void FX_CALL grDisableAllEffects( void ) {
	
};

FX_ENTRY void FX_CALL grDitherMode( GrDitherMode_t mode ) {
	
};

FX_ENTRY void FX_CALL grFogColorValue( GrColor_t fogcolor ) {
	
};

FX_ENTRY void FX_CALL grFogMode( GrFogMode_t mode ) {
	
};

FX_ENTRY void FX_CALL grFogTable( const GrFog_t ft[] ) {
	
};

FX_ENTRY void FX_CALL grLoadGammaTable( FxU32 nentries, FxU32 *red, FxU32 *green, FxU32 *blue) {
	
};

FX_ENTRY void FX_CALL grSplash(float x, float y, float width, float height, FxU32 frame) {
	//sndPlaySound("Glide3x.wav",0);
};

FX_ENTRY FxU32 FX_CALL grGet( FxU32 pname, FxU32 plength, FxI32 *params ) {
	if (plength<4) return 0;
	switch (pname) {
	case GR_BITS_DEPTH:
		*params=0;
		return 4;
	case GR_BITS_GAMMA:
		return 0;
	case GR_FOG_TABLE_ENTRIES:
		*params=0;
		return 4;
	case GR_GAMMA_TABLE_ENTRIES:
		*params=256;
		return 4;
	case GR_GLIDE_STATE_SIZE:
		*params=0;
		return 4;
	case GR_GLIDE_VERTEXLAYOUT_SIZE:
		*params=0;
		return 4;
	case GR_IS_BUSY:
		*params=0;
		return 4;
	case GR_LFB_PIXEL_PIPE:
		*params=0;
		return 4;
	case GR_MAX_TEXTURE_SIZE:
		*params=1;
		return 4;
	case GR_MAX_TEXTURE_ASPECT_RATIO:
		*params=3;
		return 4;
	case GR_MEMORY_FB:
		*params=0;
		return 4;
	case GR_MEMORY_TMU:
		*params=0;
		return 4;
	case GR_MEMORY_UMA:
		*params=VideoMemory;
		return 4;
	case GR_NON_POWER_OF_TWO_TEXTURES:
		*params=0;
		return 4;
	case GR_NUM_BOARDS:
		*params=1;
		return 4;
	/*case GR_NUM_FB:
		*params=0;
		return 4;
	case GR_NUM_SWAP_HISTORY_BUFFER:
		*params=0;
		return 4;
	case GR_NUM_TMU:
		*params=0;
		return 4;
	case GR_PENDING_BUFFERSWAPS:
		*params=0;
		return 4;
	case GR_REVISION_FB:
		*params=0;
		return 4;
	case GR_REVISION_TMU:
		*params=0;
		return 4;*/
	case GR_STATS_LINES:
		*params=LinesCount;
		return 4;
	/*case GR_STATS_PIXELS_AFUNC_FAIL:
		*params=0;
		return 4;
	case GR_STATS_PIXELS_CHROMA_FAIL:
		*params=0;
		return 4;
	case GR_STATS_PIXELS_DEPTHFUNC_FAIL:
		*params=0;
		return 4;
	case GR_STATS_PIXELS_IN:
		*params=0;
		return 4;
	case GR_STATS_PIXELS_OUT:
		*params=0;
		return 4;*/
	case GR_STATS_POINTS:
		*params=PointsCount;
		return 4;
	/*case GR_STATS_TRIANGLES_IN:
		*params=0;
		return 4;
	case GR_STATS_TRIANGLES_OUT:
		*params=0;
		return 4;
	case GR_SUPPORTS_PASSTHRU:
		*params=0;
		return 4;
	case GR_TEXTURE_ALIGN:
		*params=0;
		return 4;
*/
	default:
		*params=1;
		return 4;
	}
};

FX_ENTRY const char * FX_CALL grGetString( FxU32 pname ) {
	char *s=new char [32];
	switch (pname) {
	case GR_EXTENSION:
		return strcpy(s," ");
	case GR_HARDWARE:
		return strcpy(s,"VGA Card");
	case GR_RENDERER:
		return strcpy(s,"Glide (through GDI)");
	case GR_VENDOR:
		return strcpy(s,"Defder");
	case GR_VERSION:
		return strcpy(s,"3.x");
	default:
		return 0;
	};
};

FX_ENTRY FxI32 FX_CALL grQueryResolutions( const GrResolution *resTemplate, GrResolution *output ) {
	GrScreenResolution_t resolution;
	GrScreenRefresh_t    refresh;
	int                  numColorBuffers;
	int                  numAuxBuffers;
	int Count=0;
	for (resolution=0;resolution<=16;resolution++)
		for (refresh=0;refresh<=8;refresh++)
			for (numColorBuffers=1;numColorBuffers<=3;numColorBuffers++)
				for (numAuxBuffers=0;numAuxBuffers<=1;numAuxBuffers++)
					if ((resTemplate->resolution==GR_QUERY_ANY||resTemplate->resolution==resolution)&&
						(resTemplate->refresh==GR_QUERY_ANY||resTemplate->refresh==refresh)&&
						(resTemplate->numColorBuffers==GR_QUERY_ANY||resTemplate->numColorBuffers==numColorBuffers)&&
						(resTemplate->numAuxBuffers==GR_QUERY_ANY||resTemplate->numAuxBuffers==numAuxBuffers)) {
						if (output) {
							output[Count].refresh=refresh;
							output[Count].resolution=resolution;
							output[Count].numColorBuffers=numColorBuffers;
							output[Count].numAuxBuffers=numAuxBuffers;
						}
						Count++;
					};

	return Count*sizeof(GrResolution);
};

FX_ENTRY FxBool FX_CALL grReset( FxU32 what ) {
	return 0;	
};

FX_ENTRY GrProc FX_CALL grGetProcAddress( char *procName ) {
	return 0;	
};

FX_ENTRY void FX_CALL grEnable( GrEnableMode_t mode ) {
	
};

FX_ENTRY void FX_CALL grDisable( GrEnableMode_t mode ) {
	
};

FX_ENTRY void FX_CALL grCoordinateSpace( GrCoordinateSpaceMode_t mode ) {
	if (mode==GR_WINDOW_COORDS||mode==GR_CLIP_COORDS) CoordMode=mode;
};

FX_ENTRY void FX_CALL grDepthRange( FxFloat n, FxFloat f ) {
	
};

FX_ENTRY void FX_CALL grViewport( FxI32 x, FxI32 y, FxI32 width, FxI32 height ) {
	
};


/*
** linear frame buffer functions
*/

FX_ENTRY FxBool FX_CALL grLfbLock( GrLock_t type, GrBuffer_t buffer, GrLfbWriteMode_t writeMode,
           GrOriginLocation_t origin, FxBool pixelPipeline, 
           GrLfbInfo_t *info ) {
	return 0;	
};

FX_ENTRY FxBool FX_CALL grLfbUnlock( GrLock_t type, GrBuffer_t buffer ) {
	return 0;	
};

FX_ENTRY void FX_CALL grLfbConstantAlpha( GrAlpha_t alpha )  {
	
};

FX_ENTRY void FX_CALL grLfbConstantDepth( FxU32 depth )  {
	
};

FX_ENTRY void FX_CALL grLfbWriteColorSwizzle(FxBool swizzleBytes, FxBool swapWords)  {
	
};

FX_ENTRY void FX_CALL grLfbWriteColorFormat(GrColorFormat_t colorFormat)  {
	
};

FX_ENTRY FxBool FX_CALL grLfbWriteRegion( GrBuffer_t dst_buffer, 
                  FxU32 dst_x, FxU32 dst_y, 
                  GrLfbSrcFmt_t src_format, 
                  FxU32 src_width, FxU32 src_height, 
                  FxBool pixelPipeline,
                  FxI32 src_stride, void *src_data ) {
	return 0;	
};

FX_ENTRY FxBool FX_CALL grLfbReadRegion( GrBuffer_t src_buffer,
                 FxU32 src_x, FxU32 src_y,
                 FxU32 src_width, FxU32 src_height,
                 FxU32 dst_stride, void *dst_data ) {
	return 0;	
};

/*
** glide management functions
*/
FX_ENTRY void FX_CALL grGlideInit( void ) {
	int i;
	for (i=0;i<128;i++) ParamOffset[i]=0;
};

FX_ENTRY void FX_CALL grGlideShutdown( void ) {
	
};

FX_ENTRY void FX_CALL grGlideGetState( void *state )  {
	
};

FX_ENTRY void FX_CALL grGlideSetState( const void *state ) {
	
};

FX_ENTRY void FX_CALL grGlideGetVertexLayout( void *layout ) {
	
};

FX_ENTRY void FX_CALL grGlideSetVertexLayout( const void *layout ) {
	
};


/*
** texture mapping control functions
*/
FX_ENTRY FxU32 FX_CALL grTexCalcMemRequired(
                     GrLOD_t lodmin, GrLOD_t lodmax,
                     GrAspectRatio_t aspect, GrTextureFormat_t fmt) {
	return 0;
};

FX_ENTRY FxU32 FX_CALL grTexTextureMemRequired( FxU32 evenOdd, GrTexInfo *info ) {
	return 0;
};

FX_ENTRY FxU32 FX_CALL grTexMinAddress( GrChipID_t tmu ) {
	return 0;
};

FX_ENTRY FxU32 FX_CALL grTexMaxAddress( GrChipID_t tmu ) {
	return VideoMemory-1;
};

FX_ENTRY void FX_CALL grTexNCCTable( GrNCCTable_t table ) {
	
};

FX_ENTRY void FX_CALL grTexSource( GrChipID_t tmu,
             FxU32      startAddress,
             FxU32      evenOdd,
             GrTexInfo  *info ) {
	
};

FX_ENTRY void FX_CALL grTexClampMode(
               GrChipID_t tmu,
               GrTextureClampMode_t s_clampmode,
               GrTextureClampMode_t t_clampmode
               ) {
	
};

FX_ENTRY void FX_CALL grTexCombine(
             GrChipID_t tmu,
             GrCombineFunction_t rgb_function,
             GrCombineFactor_t rgb_factor, 
             GrCombineFunction_t alpha_function,
             GrCombineFactor_t alpha_factor,
             FxBool rgb_invert,
             FxBool alpha_invert
             ) {
	
};

FX_ENTRY void FX_CALL grTexDetailControl(
                   GrChipID_t tmu,
                   int lod_bias,
                   FxU8 detail_scale,
                   float detail_max
                   ) {
	
};

FX_ENTRY void FX_CALL grTexFilterMode(
                GrChipID_t tmu,
                GrTextureFilterMode_t minfilter_mode,
                GrTextureFilterMode_t magfilter_mode
                ) {
	
};


FX_ENTRY void FX_CALL grTexLodBiasValue(GrChipID_t tmu, float bias ) {
	
};

FX_ENTRY void FX_CALL grTexDownloadMipMap( GrChipID_t tmu,
                     FxU32      startAddress,
                     FxU32      evenOdd,
                     GrTexInfo  *info ) {
	
};

FX_ENTRY void FX_CALL grTexDownloadMipMapLevel( GrChipID_t        tmu,
                          FxU32             startAddress,
                          GrLOD_t           thisLod,
                          GrLOD_t           largeLod,
                          GrAspectRatio_t   aspectRatio,
                          GrTextureFormat_t format,
                          FxU32             evenOdd,
                          void              *data ) {
	
};

FX_ENTRY FxBool FX_CALL grTexDownloadMipMapLevelPartial( GrChipID_t        tmu,
                                 FxU32             startAddress,
                                 GrLOD_t           thisLod,
                                 GrLOD_t           largeLod,
                                 GrAspectRatio_t   aspectRatio,
                                 GrTextureFormat_t format,
                                 FxU32             evenOdd,
                                 void              *data,
                                 int               start,
                                 int               end ) {
	return 0;	
};

FX_ENTRY void FX_CALL grTexDownloadTable( GrTexTable_t type, 
                    void         *data ) {
	
};

FX_ENTRY void FX_CALL grTexDownloadTablePartial( GrTexTable_t type, 
                           void         *data,
                           int          start,
                           int          end ) {
	
};

FX_ENTRY void FX_CALL grTexMipMapMode( GrChipID_t     tmu, 
                 GrMipMapMode_t mode,
                 FxBool         lodBlend ) {
	
};

FX_ENTRY void FX_CALL grTexMultibase( GrChipID_t tmu,
                FxBool     enable ) {
	
};

FX_ENTRY void FX_CALL grTexMultibaseAddress( GrChipID_t       tmu,
                       GrTexBaseRange_t range,
                       FxU32            startAddress,
                       FxU32            evenOdd,
                       GrTexInfo        *info ) {
	
};


/*
** Gamma functions
*/

FX_ENTRY void FX_CALL guGammaCorrectionRGB( FxFloat red, FxFloat green, FxFloat blue ) {
	
};

/*
** fog stuff
*/
FX_ENTRY float FX_CALL guFogTableIndexToW( int i ) {
	return 0;
};

FX_ENTRY void FX_CALL guFogGenerateExp( GrFog_t *fogtable, float density ) {
	
};

FX_ENTRY void FX_CALL guFogGenerateExp2( GrFog_t *fogtable, float density ) {
	
};

FX_ENTRY void FX_CALL guFogGenerateLinear(GrFog_t *fogtable, float nearZ, float farZ ) {
	
};

/*
** hi-level texture manipulation tools.
*/
FX_ENTRY FxBool FX_CALL gu3dfGetInfo( const char *filename, Gu3dfInfo *info ) {
	return 0;	
};

FX_ENTRY FxBool FX_CALL gu3dfLoad( const char *filename, Gu3dfInfo *data ) {
	return 0;
};
